<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\EventBundle\Entity;

use Chill\MainBundle\Entity\Center;
use Chill\MainBundle\Entity\HasCenterInterface;
use Chill\MainBundle\Entity\HasScopeInterface;
use Chill\MainBundle\Entity\Scope;
use Chill\MainBundle\Entity\User;
use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;

/**
 * Class Event.
 *
 * @ORM\Entity(repositoryClass="Chill\EventBundle\Repository\EventRepository")
 *
 * @ORM\Table(name="chill_event_event")
 *
 * @ORM\HasLifecycleCallbacks
 */
class Event implements HasCenterInterface, HasScopeInterface
{
    /**
     * @ORM\ManyToOne(targetEntity="Chill\MainBundle\Entity\Center")
     */
    private ?\Chill\MainBundle\Entity\Center $center = null;

    /**
     * @ORM\ManyToOne(targetEntity="Chill\MainBundle\Entity\Scope")
     */
    private ?\Chill\MainBundle\Entity\Scope $circle = null;

    /**
     * @ORM\Column(type="datetime")
     */
    private ?\DateTime $date;

    /**
     * @ORM\Id
     *
     * @ORM\Column(name="id", type="integer")
     *
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    private ?int $id = null;

    /**
     * @ORM\ManyToOne(targetEntity="Chill\MainBundle\Entity\User")
     */
    private ?\Chill\MainBundle\Entity\User $moderator = null;

    /**
     * @ORM\Column(type="string", length=150)
     */
    private ?string $name = null;

    /**
     * @var Collection<Participation>
     *
     * @ORM\OneToMany(
     *     targetEntity="Chill\EventBundle\Entity\Participation",
     * mappedBy="event")
     */
    private Collection $participations;

    /**
     * @ORM\ManyToOne(targetEntity="Chill\EventBundle\Entity\EventType")
     */
    private ?\Chill\EventBundle\Entity\EventType $type = null;

    /**
     * Event constructor.
     */
    public function __construct()
    {
        $this->participations = new ArrayCollection();
    }

    /**
     * Add participation.
     *
     * @return Event
     */
    public function addParticipation(Participation $participation)
    {
        $this->participations[] = $participation;

        return $this;
    }

    /**
     * @return Center
     */
    public function getCenter()
    {
        return $this->center;
    }

    /**
     * @return Scope
     */
    public function getCircle()
    {
        return $this->circle;
    }

    /**
     * Get date.
     *
     * @return \DateTime
     */
    public function getDate()
    {
        return $this->date;
    }

    /**
     * Get id.
     *
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    public function getModerator(): null|User
    {
        return $this->moderator;
    }

    /**
     * Get label.
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return Collection<int, Participation>
     */
    public function getParticipations(): Collection
    {
        return new ArrayCollection(iterator_to_array($this->getParticipationsOrdered()));
    }

    /**
     * Sort Collection of Participations.
     */
    public function getParticipationsOrdered(): \ArrayIterator|\Traversable
    {
        $iterator = iterator_to_array($this->participations->getIterator());

        uasort($iterator, static fn ($first, $second) => strnatcasecmp((string) $first->getPerson()->getFirstName(), (string) $second->getPerson()->getFirstName()));

        return new \ArrayIterator($iterator);
    }

    /**
     * @deprecated
     *
     * @return Scope
     */
    public function getScope()
    {
        return $this->getCircle();
    }

    /**
     * @return EventType
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * Remove participation.
     */
    public function removeParticipation(Participation $participation)
    {
        $this->participations->removeElement($participation);
    }

    /**
     * @return $this
     */
    public function setCenter(Center $center)
    {
        $this->center = $center;

        return $this;
    }

    /**
     * @return $this
     */
    public function setCircle(Scope $circle)
    {
        $this->circle = $circle;

        return $this;
    }

    /**
     * Set date.
     *
     * @return Event
     */
    public function setDate(\DateTime $date)
    {
        $this->date = $date;

        return $this;
    }

    public function setModerator(User $moderator): self
    {
        $this->moderator = $moderator;

        return $this;
    }

    /**
     * Set label.
     *
     * @param string $label
     *
     * @return Event
     */
    public function setName($label)
    {
        $this->name = $label;

        return $this;
    }

    /**
     * @return $this
     */
    public function setType(EventType $type)
    {
        $this->type = $type;

        return $this;
    }
}
