<?php

declare(strict_types=1);

/*
 * Chill is a software for social workers
 *
 * For the full copyright and license information, please view
 * the LICENSE file that was distributed with this source code.
 */

namespace Chill\PersonBundle\Tests\Controller;

use Chill\MainBundle\Test\PrepareClientTrait;
use Chill\PersonBundle\Entity\Person;
use Chill\PersonBundle\Entity\Relationships\Relation;
use Doctrine\ORM\EntityManagerInterface;
use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Symfony\Component\HttpFoundation\Request;

/**
 * @internal
 *
 * @coversNothing
 */
final class RelationshipApiControllerTest extends WebTestCase
{
    use PrepareClientTrait;

    /**
     * A cache for all relations.
     *
     * @var array|Relation[]|null
     */
    private ?array $relations = null;

    protected function tearDown(): void
    {
        self::ensureKernelShutdown();
    }

    public function personProvider(): array
    {
        self::bootKernel();
        $em = self::$container->get(EntityManagerInterface::class);
        $countPersons = $em->createQueryBuilder()
            ->select('count(p)')
            ->from(Person::class, 'p')
            ->join('p.centerCurrent', 'center_current')
            ->join('center_current.center', 'c')
            ->where('c.name LIKE :name')
            ->setParameter('name', 'Center A')
            ->getQuery()
            ->getSingleScalarResult();
        $person = $em->createQueryBuilder()
            ->select('p')
            ->from(Person::class, 'p')
            ->join('p.centerCurrent', 'center_current')
            ->join('center_current.center', 'c')
            ->where('c.name LIKE :name')
            ->setParameter('name', 'Center A')
            ->getQuery()
            ->setMaxResults(1)
            ->setFirstResult(\random_int(0, $countPersons - 1))
            ->getSingleResult();

        self::ensureKernelShutdown();

        return [
            [$person->getId()],
        ];
    }

    public function relationProvider(): array
    {
        self::bootKernel();
        $em = self::$container->get(EntityManagerInterface::class);
        $countPersons = $em->createQueryBuilder()
            ->select('count(DISTINCT p)')
            ->from(Person::class, 'p')
            ->join('p.centerCurrent', 'center_current')
            ->join('center_current.center', 'c')
            ->where('c.name LIKE :name')
            ->setParameter('name', 'Center A')
            ->getQuery()
            ->getSingleScalarResult();

        $persons = $em->createQueryBuilder()
            ->select('p')
            ->from(Person::class, 'p')
            ->join('p.centerCurrent', 'center_current')
            ->join('center_current.center', 'c')
            ->where('c.name LIKE :name')
            ->setParameter('name', 'Center A')
            ->getQuery()
            ->setMaxResults(2)
            ->setFirstResult(\random_int(0, $countPersons - 1))
            ->getResult();

        self::ensureKernelShutdown();

        return [
            [$persons[0]->getId(), $persons[1]->getId(), $this->getRandomRelation($em)->getId(), true],
        ];
    }

    /**
     * @dataProvider personProvider
     */
    public function testGetRelationshipByPerson(mixed $personId)
    {
        self::ensureKernelShutdown();
        $client = $this->getClientAuthenticated();
        $client->request(Request::METHOD_GET, sprintf('/api/1.0/relations/relationship/by-person/%d.json', $personId));

        $response = $client->getResponse();
        $this->assertEquals(200, $response->getStatusCode(), 'Test to see that API response returns a status code 200');
    }

    /**
     * @dataProvider relationProvider
     */
    public function testPostRelationship(mixed $fromPersonId, mixed $toPersonId, mixed $relationId, mixed $isReverse): void
    {
        $client = $this->getClientAuthenticated();

        $client->request(
            Request::METHOD_POST,
            '/api/1.0/relations/relationship.json',
            [],
            [],
            [],
            \json_encode([
                'type' => 'relationship',
                'fromPerson' => ['id' => $fromPersonId, 'type' => 'person'],
                'toPerson' => ['id' => $toPersonId, 'type' => 'person'],
                'relation' => ['id' => $relationId, 'type' => 'relation'],
                'reverse' => $isReverse,
            ], JSON_THROW_ON_ERROR)
        );

        $response = $client->getResponse();
        $this->assertEquals(200, $response->getStatusCode());
    }

    private function getRandomRelation(EntityManagerInterface $em): Relation
    {
        if (null === $this->relations) {
            $this->relations = $em->getRepository(Relation::class)
                ->findAll();
        }

        return $this->relations[\array_rand($this->relations)];
    }
}
